/* 
 * $Id: beetooclient.h,v 1.5 2003/06/14 17:28:58 mikelima Exp $
 *
 * B-II KWin Client
 *
 * Changes:
 * 	Customizable button positions by Karol Szwed <gallium@kde.org>
 */ 

#ifndef __B2CLIENT_H
#define __B2CLIENT_H

#include <qvariant.h>
#include <qdatetime.h>
#include <qbitmap.h>
#include <kpixmap.h>
#include <kwin/client.h>
#include <kwin/kwinbutton.h>

class QSpacerItem;
class QBoxLayout;
class QGridLayout;

namespace B2 {

using namespace KWinInternal;

class B2Button : public KWinButton
{
    Q_OBJECT
public:
    B2Button(Client *_client=0, 
             QWidget *parent=0, const QString& tip=NULL, int pixIndex=0);

    void setBg(const QColor &c) { bg = c; }
    void setPixmaps(int button_id);
    void setToggle() { setToggleType(Toggle); }
    void setActive(bool on) { setOn(on); }
    void setUseMiniIcon() { useMiniIcon = true; }
    QSize sizeHint() const;
    QSizePolicy sizePolicy() const;
protected:
    virtual void drawButton(QPainter *p);
    void drawButtonLabel(QPainter *) {}
    
    void mousePressEvent(QMouseEvent* e);
    void mouseReleaseEvent(QMouseEvent* e);
private:
    void enterEvent(QEvent *e);
    void leaveEvent(QEvent *e);
    
    KPixmap *icon[6];
    QColor bg; //only use one color (the rest is pixmap) so forget QPalette ;)
    bool useMiniIcon;
    bool hover;
    Client* client;

public:
    int last_button;    
};

class B2Client;

class B2Titlebar : public QWidget
{
    Q_OBJECT
public:
    B2Titlebar(B2Client *parent);
    bool isFullyObscured() const {return isfullyobscured;}
    void recalcBuffer();
    QSpacerItem *captionSpacer;
    QSpacerItem *endLabel[2];
protected:
    void paintEvent( QPaintEvent* );
    bool x11Event(XEvent *e);
    void mouseDoubleClickEvent( QMouseEvent * );
    void mousePressEvent( QMouseEvent * );
    void mouseReleaseEvent( QMouseEvent * );
    void mouseMoveEvent(QMouseEvent *);
    void resizeEvent(QResizeEvent *ev);
    
    QString oldTitle;
    KPixmap titleBuffer;
    bool set_x11mask;
    bool isfullyobscured;
    bool shift_move;
    QPoint moveOffset;
    B2Client *client;
private:
    void drawTitleBar(QPainter &p, QRect &t, bool state);
};

class B2Client : public Client
{
    Q_OBJECT
    friend class B2Titlebar;
public:
    B2Client( Workspace *ws, WId w, QWidget *parent=0, const char *name=0 );
    ~B2Client(){;}
    void unobscureTitlebar();
    void titleMoveAbs(int new_ofs);
    void titleMoveRel(int xdiff);

    // transparent stuff
    virtual void drawbound(const QRect& geom);
    virtual void clearbound();

protected:
    void resizeEvent( QResizeEvent* );
    void paintEvent( QPaintEvent* );
    void showEvent( QShowEvent* );
    void windowWrapperShowEvent( QShowEvent* );
    void captionChange( const QString& name );
    void stickyChange(bool on);
    void activeChange(bool on);
    void maximizeChange(bool m);
    void iconChange();
    void doShape();
    MousePosition mousePosition( const QPoint& p ) const;
private slots:
    void menuButtonPressed();
    void slotReset();
    void maxButtonClicked();
private:
    void addButtons(const QString& s, const QString tips[], 
                    B2Titlebar* tb, QBoxLayout* titleLayout);
    void positionButtons();
    void calcHiddenButtons();
    bool mustDrawHandle() const { return draw_handle && isResizable() && !isMenu(); }
    
    enum ButtonType{ 
	BtnMenu=0, BtnSticky, BtnIconify, BtnMax, BtnClose, BtnHelp, BtnCount
    };
    B2Button* button[BtnCount];
    QGridLayout *g;
    QSpacerItem *spacer; // Bottom border spacer
    QSpacerItem *vspacer1; // Left and right border spacer
    QSpacerItem *vspacer2; // Left and right border spacer
    B2Titlebar *titlebar;
    int bar_x_ofs;
    int in_unobs;
    QTime time;
    bool draw_handle;
};

};

#endif
