/* 	
 * $Id: config.cpp,v 1.2 2003/05/18 20:07:33 mikelima Exp $
 *
 *	This file contains the B2 configuration widget
 *
 *	Copyright (c) 2001
 *		Karol Szwed <gallium@kde.org>
 *		http://gallium.n3.net/
 *	Copyright 2003
 *		Luciano Montanaro <mikelima@virgilio.it>
 */

#include "config.h"
#include <qstring.h>
#include <qregexp.h>
#include <qhbox.h>
#include <qhgroupbox.h>
#include <qbitmap.h>
#include <qpaintdevice.h>
#include <qimage.h>
#include <qpainter.h>
#include <kglobal.h>
#include <qwhatsthis.h>
#include <klocale.h>
#include <kpixmap.h>
#include <kimageeffect.h>
#include <kstandarddirs.h>


extern "C"
{ 
    QObject* allocate_config(KConfig* conf, QWidget* parent) 
    { 
	return(new B2Config(conf, parent)); 
    } 
}


/* NOTE: 
 * 'conf' is a pointer to the kwindecoration modules open kwin config, and is
 * by default set to the "Style" group.
 *
 * 'parent' is the parent of the QObject, which is a VBox inside the Configure
 * tab in kwindecoration
 */

B2Config::B2Config(KConfig* conf, QWidget* parent) : QObject(parent)
{
    KGlobal::locale()->insertCatalogue("kwin_beetoo_config");
    b2Config = new KConfig("kwinbeetoorc");
    gb = new QGroupBox(1, Qt::Horizontal, i18n("Decoration Settings"), parent);
    colorBorderCb = new QCheckBox(
	    i18n("Draw window frames using &titlebar colors"), gb);
    QWhatsThis::add(colorBorderCb, 
	    i18n("When selected, the window decoration borders are drawn "
		"using the titlebar colors. Otherwise, they are drawn using "
		"normal border colors instead."));

    // Grab Handle
    showGrabHandleCb = new QCheckBox(
	    i18n("Draw g&rab handle below windows"), gb);
    QWhatsThis::add(showGrabHandleCb, 
	    i18n("When selected, decorations are drawn with a \"grab handle\" " 
		 "below windows. Otherwise, no grab bar is drawn."));

    // Frame Thickness
    QHBox *sliderBox = new QHBox(gb);
    QLabel *thicknessLabel = new QLabel(sliderBox);
    thicknessLabel->setText(i18n("Frame width:"));
    thicknessSl = new QSlider(1, 10, 1, 5, QSlider::Horizontal, sliderBox);
    thicknessSl->setTickmarks(QSlider::Below);
    QWhatsThis::add(thicknessSl, 
	    i18n("By moving this slider back and forth, you can adjust the "
		 "width of the window frame to a value of your liking."));
    
    // Tile selection support
    QHBox *boxTile = new QHBox(gb);
    QLabel *titleEffectLabel = new QLabel(boxTile);
    titleEffectLabel->setText(i18n("Title Bar effect:"));
    titleEffectCb = new QComboBox(boxTile);
    loadEffectPixmaps();
    // Double click menu option support
    actionsGB = new QHGroupBox(i18n("Actions Settings"), parent);
    QLabel *menuDblClickLabel = new QLabel(actionsGB);
    menuDblClickLabel->setText(i18n("Double Click on Menu Button:"));
    menuDblClickOp = new QComboBox(actionsGB);
    menuDblClickOp->insertItem(i18n("Do Nothing"));
    menuDblClickOp->insertItem(i18n("Iconify Window"));
    menuDblClickOp->insertItem(i18n("Close Window"));
    
    QWhatsThis::add(menuDblClickOp, 
	    i18n("An action can be associated to a double click " 
		 "of the menu button. Leave it to none if in doubt."));
    // Load configuration options
    load(conf);

    // Ensure we track user changes properly
    connect(colorBorderCb, SIGNAL(clicked()), 
		    this, SLOT(slotSelectionChanged()));
    connect(showGrabHandleCb, SIGNAL(clicked()), 
		    this, SLOT(slotSelectionChanged()));
    connect(titleEffectCb, SIGNAL(activated(int)), 
		    this, SLOT(slotSelectionChanged()));
    connect(menuDblClickOp, SIGNAL(activated(int)), 
		    this, SLOT(slotSelectionChanged()));
    connect(thicknessSl, SIGNAL(valueChanged(int)), 
		    this, SLOT(slotSelectionChanged()));
    // Make the widgets visible in kwindecoration
    gb->show();
    actionsGB->show();
}

void B2Config::loadEffectPixmaps(void)
{
    // Find gradient colors
    KConfig *c = KGlobal::config();
    KConfigGroupSaver cgs( c, QString::fromLatin1("WM") );
    QColor activeBackground = c->readColorEntry("activeBackground");
    QColor activeBlend = c->readColorEntry("activeBlend");
    QColor inactiveBackground = c->readColorEntry("inactiveBackground");
    QColor inactiveBlend = c->readColorEntry("inactiveBlend");
    
    // Find available effects
    QStringList resList = KGlobal::dirs()->findAllResources("data", 
	    "kwin/beetoo/*.b2effect");
    QStringList::ConstIterator it;

    // Create preview pixmaps
    QImage img;
    QPixmap effect, activeEffect[2], inactiveEffect[2];
    effect.resize(180, 20);
    effect.setMask(QBitmap(180, 20));
    bitBlt((QPaintDevice *)effect.mask(), 0, 0, 
	    (QPaintDevice *)effect.mask(), 0, 0, 180, 20, Qt::ClearROP);
    bitBlt((QPaintDevice *)effect.mask(), 0, 1, 
	    (QPaintDevice *)effect.mask(), 0, 0, 180, 18, Qt::SetROP);
    
    int endWidth[2];
    bool endsDefined;
    
    for (it = resList.begin(); it != resList.end(); ++it) {
	effects.append(*it);
	img.load(*it);
        QString b2Segments = img.text("b2segments");
        if (!b2Segments.isEmpty()) {
	    endsDefined = true;
    	    endWidth[0] = b2Segments.section(',', 0, 0).toInt();
    	    endWidth[1] = b2Segments.section(',', 1, 1).toInt();
        } else {
	    endsDefined = false;
	    endWidth[0] = endWidth[1] = 3;
        }
	
	QPainter p;
	// Create active tile
	KImageEffect::flatten(img, activeBlend, activeBackground, 32);
	if (endsDefined) {
	    activeEffect[0].resize(endWidth[0], 18);
	    p.begin(&activeEffect[0]);
	    p.drawImage(0, 0, img);
	    p.end();
	    activeEffect[1].resize(
		    img.width() - endWidth[0] - endWidth[1], 18);
	    p.begin(&activeEffect[1]);
	    p.drawImage(0, 0, img, endWidth[0], 0);
	    p.end();
	} else {
	    activeEffect[1].convertFromImage(img);
	}
	// Create inactive tile
	img.load(*it);
	KImageEffect::flatten(img, inactiveBlend, inactiveBackground, 32);
	if (endsDefined) {
	    inactiveEffect[0].resize(endWidth[1], 18);
	    p.begin(&inactiveEffect[0]);
	    p.drawImage(0, 0, img, img.width() - endWidth[1]);
	    p.end();
	    inactiveEffect[1].resize(
		    img.width() - endWidth[0] - endWidth[1], 18);
	    p.begin(&inactiveEffect[1]);
	    p.drawImage(0, 0, img, endWidth[0], 0);
	    p.end();
	} else {
	    inactiveEffect[1].convertFromImage(img);
	}

	// Compose tiles in final preview pixmap
        p.begin(&effect);
	if (endsDefined) {
	    p.drawPixmap(0, 1, activeEffect[0]);
	    p.drawTiledPixmap(endWidth[0], 1, 
		    90 - endWidth[0], 18, activeEffect[1]);
	    p.drawTiledPixmap(90, 1, 90 - endWidth[1], 18, inactiveEffect[1]);
	    p.drawPixmap(180 - endWidth[1] - 1, 1, inactiveEffect[0]);
	} else {
	    p.drawTiledPixmap(0, 1, 90, 18, activeEffect[1]);
	    p.drawTiledPixmap(90, 1, 90, 18, inactiveEffect[1]);
	}
	p.end();
   	titleEffectCb->insertItem(effect);
    }
}

B2Config::~B2Config()
{
    delete b2Config;
    delete gb;
    delete actionsGB;
}


void B2Config::slotSelectionChanged()
{
    emit changed();
}

// Loads the configurable options from the kwinrc config file
// It is passed the open config from kwindecoration to improve efficiency
void B2Config::load( KConfig* /*conf*/ )
{
    b2Config->setGroup("General");
    bool override = b2Config->readBoolEntry( "UseTitleBarBorderColors", false );
    colorBorderCb->setChecked( override );
    
    override = b2Config->readBoolEntry( "DrawGrabHandle", true );
    showGrabHandleCb->setChecked(override);

    QString fxName = b2Config->readEntry("TitleEffectName");
    
    uint fx = 0;
    
    for (uint i = 0; i < effects.count(); i++) {
	if (effects[i] == fxName) {
	    fx = i;
	    break;
	}
    }

    titleEffectCb->setCurrentItem(fx);
    
    QString returnString = b2Config->readEntry( "MenuButtonDoubleClickOperation", "NoOp");
    
    int op;
    if (returnString == "CloseWindow") {
	op = 2;
    } else if (returnString == "IconifyWindow") {
	op = 1;
    } else {
	op = 0;
    }

    menuDblClickOp->setCurrentItem(op);

    op = b2Config->readUnsignedNumEntry("FrameThickness", 5);
    thicknessSl->setValue(op);
}

static QString opToString(int op) 
{
    switch (op) {
    case 1:
	    return "IconifyWindow";
    case 2:
	    return "CloseWindow";
    case 0:
    default:
	    return "NoOp";
    }
}

// Saves the configurable options to the kwinrc config file
void B2Config::save( KConfig* /*conf*/ )
{
    b2Config->setGroup("General");
    b2Config->writeEntry("UseTitleBarBorderColors", 
	    colorBorderCb->isChecked());
    b2Config->writeEntry("DrawGrabHandle", showGrabHandleCb->isChecked());
    b2Config->writeEntry("TitleEffectName", 
	    effects[titleEffectCb->currentItem()]);
    b2Config->writeEntry("MenuButtonDoubleClickOperation", 
	    opToString(menuDblClickOp->currentItem()));
    b2Config->writeEntry("FrameThickness", thicknessSl->value());
    // Ensure others trying to read this config get updated
    b2Config->sync();
}

// Sets UI widget defaults which must correspond to style defaults
void B2Config::defaults()
{
    colorBorderCb->setChecked(false);
    showGrabHandleCb->setChecked(true);
    titleEffectCb->setCurrentItem(0);
    menuDblClickOp->setCurrentItem(0);
    thicknessSl->setValue(5);
}

#include "config.moc"
/* vim: set ts=8 sw=4 : */
